;-------------------------------------------------------------------------
;			UART driver and interrupot handler
;-------------------------------------------------------------------------
;.equ	bauddivider = (xtal / baudrate + 8) / 16 - 1
;-------------------------------------------------------------------------
;				initialization
;-------------------------------------------------------------------------
uart_init:
;	ldi	a0, low(bauddivider)		;set baud rate
;	ldi	a1, high(bauddivider)
;
;input: a1:a0 = baud rate divider
set_baud:
	out	UBRRL, a0
	out	UBRRH, a1

	ldi	a0, 0				;no U2X, MPCM
	out	UCSRA, a0
	ldi	a0, 1<<URSEL^1<<UCSZ1^1<<UCSZ0	;8 Bit
	out	UCSRC, a0

	mov	rx_ptr, xl			;initialize rx buffer

	ldi	a0, 1<<RXEN^1<<TXEN^1<<RXCIE	;enable RX, TX
	out	UCSRB, a0
	ret
;-------------------------------------------------------------------------
;				Receive Interrupt
;-------------------------------------------------------------------------
URXCint:
	ldi	xh, high(rx_buff)
	in	rx_data, UDR
	st	x+, rx_data
	reti
;-------------------------------------------------------------------------
;				Receive Byte
;-------------------------------------------------------------------------
;output: a0 = byte
;
getchar:				;wait until one byte received
	cp	rx_ptr, xl
	breq	getchar
	mov	yl, rx_ptr
	ldi	yh, high(rx_buff)
	ld	a0, y
	inc	rx_ptr
	ret
;-------------------------------------------------------------------------
;				Check if command received
;-------------------------------------------------------------------------
;Output: C = 1: command received
;	Y = command start
;	rx_buff_outl = after command end
;
rx_command:
	mov	yl, rx_ptr
_rxc1:	cp	yl, xl
	breq	_rxc3
	ldi	yh, high(rx_buff)
	ld	a0, y+
	subi	a0, key_enter
	breq	_rxc2
	subi	a0, key_linefeed - key_enter
	brne	_rxc1
_rxc2:	mov	a1, yl
	st	-y, a0			;end of command = 0
	cpse	yl, rx_ptr		;ignore empty command
	sec
	mov	yl, rx_ptr		;point on start of command
	mov	rx_ptr, a1		;point after command
_rxc3:	ret
;-------------------------------------------------------------------------
;				transmit byte
;-------------------------------------------------------------------------
;input: a0 = byte
;
putchar:				;wait until tx buffer empty
	sbis	UCSRA, UDRE
	rjmp	putchar
	out	UDR, a0
	ret
;-------------------------------------------------------------------------
